"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const uuiderror_1 = require("./uuiderror");
/* tslint:disable */
class UUID {
    /**
     * Creates a UUID string.
     * @returns {string} A UUID string.
     */
    static createUUID() {
        // your favourite UUID generation function could go here
        // ex: http://stackoverflow.com/a/8809472/188246
        let d = new Date().getTime();
        if (window.performance && typeof window.performance.now === 'function') {
            d += performance.now(); // use high-precision timer if available
        }
        return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, (c) => {
            let r = ((d + Math.random() * 16) % 16) | 0;
            d = Math.floor(d / 16);
            return (c === 'x' ? r : (r & 0x3) | 0x8).toString(16);
        });
    }
    /**
     * Check if the given test string is a valid uuid string.
     * https://stackoverflow.com/questions/7905929/how-to-test-valid-uuid-UUID/13653180#13653180
     * @param {string} uuidTestString
     * @returns {boolean} True if it is a valid uuid string, otherwise false.
     */
    static isValidUUID(uuidTestString) {
        return /^[0-9a-f]{8}-[0-9a-f]{4}-[1-5][0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}$/i.test(uuidTestString);
    }
    /**
     * Get the uuid string, if the passed string is a valid uuid string.
     * @param {string} str
     * @returns {string} The uuid string, if it is a valid uuid, otherwise null.
     */
    static getValidUUID(str) {
        if (UUID.isValidUUID(str))
            return str;
        return null;
    }
    /**
     * Get a dash free UUID.
     * @param {UUID} uuid
     * @returns {String} A dash free UUID.
     */
    static getDashFreeUUID(uuid) {
        return uuid.toString().replace(/-/g, '');
    }
    static getDashContainedUUID(dashFreeUuid) {
        // 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'
        return new UUID(dashFreeUuid.slice(0, 7) +
            '-' +
            dashFreeUuid.slice(8, 11) +
            '-' +
            dashFreeUuid.slice(12, 15) +
            '-' +
            dashFreeUuid.slice(16, 19) +
            '-' +
            dashFreeUuid.slice(20, 31));
    }
    /**
     * Constructs a new UUID from the given parameter, if it is a valid UUID string.
     * If no parameter is passed, a UUID will be generated.
     * @param {string} [str]
     * @throws {UUIDError}
     */
    constructor(str) {
        if (str) {
            if (!UUID.isValidUUID(str)) {
                throw new uuiderror_1.UUIDError(str);
            }
            else {
                this.str = str;
            }
        }
        else {
            this.str = UUID.createUUID();
        }
    }
    /**
     * Checks if the given UUID string or class is equal to this UUID.
     * @param {string|UUID} uuid
     * @returns {boolean}
     */
    equals(uuid) {
        if (uuid instanceof UUID) {
            return this.str === uuid.toString();
        }
        else if (typeof uuid === 'string') {
            return this.str === uuid;
        }
        else {
            return false;
        }
    }
    getDashFreeUUID() {
        return UUID.getDashFreeUUID(this);
    }
    /**
     * Get the UUID in string representation.
     * @returns {string} UUID as string.
     */
    toString() {
        return this.str;
    }
}
exports.UUID = UUID;
